@@ -1,3 +1,5 @@
+# type: ignore
+
 # flake8:noqa E501  Ignore long lines, some test cases are just inherently long
 
 from typing import Iterable, Type
@@ -6,7 +8,7 @@ from remerkleable.complex import Container, Vector, List
 from remerkleable.basic import boolean, bit, byte, uint8, uint16, uint32, uint64, uint128, uint256
 from remerkleable.bitfields import Bitvector, Bitlist
 from remerkleable.byte_arrays import ByteVector, ByteList
-from remerkleable.core import TypeDef, View, ObjType
+from remerkleable.core import View, ObjType
 from hashlib import sha256
 
 import json
@@ -362,11 +364,6 @@ def test_value_byte_length(name: str, typ: Type[View], value: View, serialized:
     assert value.value_byte_length() == len(bytes.fromhex(serialized))
 
 
-@pytest.mark.parametrize("name, typ, value, serialized, root, obj", test_data)
-def test_typedef(name: str, typ: Type[View], value: View, serialized: str, root: str, obj: ObjType):
-    assert issubclass(typ, TypeDef)
-
-
 @pytest.mark.parametrize("name, typ, value, serialized, root, obj", test_data)
 def test_serialize(name: str, typ: Type[View], value: View, serialized: str, root: str, obj: ObjType):
     stream = io.BytesIO()
